/*
 * Project name:
    PIC32 goes into sleep mode and wakes up via RTC Alarm by interrupt
 * Generated by:
    Wolf Alexander
 * Date of creation
    10.02.2015

 * Test configuration:
     MCU:             P32MX795F512L
     Dev.Board:       Mikromedia_Plus_for_PIC32MX7

     Oscillator:      8 MHz ! COSC, No PLL
     SW:              mikroC PRO for PIC32

 * Note
     This is the basic structure of a power saving example. Look at project settings!
     Pic32 starts and works at 8Mhz without PLL for low current consumption. When shutting down,
     8Mhz crystal and all Modules are deaktivated but SOSC is enabled to be able to wake up CPU by RTC.
     Check section 6, 10 , 29 in Reference Manual and/or http://www.microchip.com/forums
     
     - Put Port D low switch as LED
     - enable UART on SW 12
     - change alarm time settings in AlarmTime struct
     - Measure with a OSCI at the CPU Crystals to see what happens
    
    ---> may this help others to save time
*/

sbit R_LED at LATD1_bit;
sbit R_LED_Direction at TRISD1_bit;
sbit G_LED at LATD2_bit;
sbit G_LED_Direction at TRISD2_bit;


void RTC_Init();
void RTC_writeUart();

void disablePIC32modules();
void enablePIC32();
void shutDownPIC32();



void RTCC_Interrupt() iv IVT_RTCC ilevel 7 ics ICS_AUTO {

  enablePIC32();
  UART2_Init(9600);   // UART was deaactivated so it has to be initiated again
                      // same for all other modules which are needed after shut down
  UART2_Write_text("\x0DAwake and Reseting!!!\x0D");
  RTCCIF_bit = 0;
  Reset();

}

void enablePIC32(){

  SYSKEY = 0x0;
  SYSKEY = 0xAA996655;
  SYSKEY = 0x556699AA;

  CLKLOCK_bit = 0;      // enable COSC
  SOSCEN_bit = 1;
  NOSC2_bit = 0;
  NOSC1_bit = 1;
  NOSC0_bit = 0;
  OSWEN_bit = 1;

  SLPEN_bit = 0;
  SYSKEY = 0x0;
}

void shutDownPIC32(){

  SYSKEY = 0x0;
  SYSKEY = 0xAA996655;
  SYSKEY = 0x556699AA;

  CLKLOCK_bit = 0;      // enable  SOSC, disable COSC
  SOSCEN_bit = 1;
  NOSC2_bit = 1;
  NOSC1_bit = 0;
  NOSC0_bit = 0;
  OSWEN_bit = 1;

  SLPEN_bit = 1;
  SYSKEY = 0x0;
  asm {wait};            // wait for interrupts
}

////////////////////////////////////////////////////////////////////////////////
// Tunrning off all Module for low Power consumtpition SLEEP mode             //
//        PIC32MX795F512L                                                     //
////////////////////////////////////////////////////////////////////////////////
void disablePIC32modules(){

  // MISC //
  C1CONbits.ON = 0; // CAN
  C2CONbits.ON = 0;
  CNCONbits.ON = 0; // CHANGE NOTICE AND PULL-UP REGISTER MAP

  ETHCON1bits.ON = 0;

  // 8. Interrupts
 /*
  IEC1 = 0x0000;   // must not disabled because of wake up by interrupt
  IEC2 = 0x0000;
  asm  {di};
 */
  
  // 9. Watchdog Timer and Power-up Timer
  WDTCONbits.ON = 0;

  // 12. I/O Ports
  TRISA = 0x0000;    // as OUTPUT
  TRISB = 0x0000;
  TRISC = 0x0000;
  TRISD = 0x0000;
  TRISE = 0x0000;
  TRISG = 0x0000;

  PORTA = 0x0000;    // as OUTPUT LOW
  PORTB = 0x0000;
  PORTC = 0x0000;
  PORTD = 0x0000;
  PORTE = 0x0000;
  PORTF = 0x0000;
  PORTG = 0x0000;

  // 13. Parallel Master Port
  PMCONbits.ON  = 0;

  // 14. Timers
  T1CONbits.ON = 0;
  T2CONbits.ON = 0;
  T3CONbits.ON = 0;
  T4CONbits.ON = 0;
  T5CONbits.ON = 0;

  // 15. Input Capture
  IC1CONbits.ON = 0;
  IC2CONbits.ON = 0;
  IC3CONbits.ON = 0;
  IC4CONbits.ON = 0;
  IC5CONbits.ON = 0;

  // 16. Output Compare
  OC1CONbits.ON = 0;
  OC2CONbits.ON = 0;
  OC3CONbits.ON = 0;
  OC4CONbits.ON = 0;
  OC5CONbits.ON = 0;

  // 17. ADC
  AD1CON1bits.ON = 0;

  // 19. Comparator
  CM1CONbits.ON = 0;   // COMPARATOR REGISTER MAP
  CM2CONbits.ON = 0;

  // 20. Comparator Voltage Reference Control Register
  CVRCONbits.ON = 0;   // COMPARATOR VOLTAGE REFERENCE REGISTER MAP

  // 21. UART Modules
  U1MODEbits.ON = 0;
  U2MODEbits.ON = 0;
  U3MODEbits.ON = 0;
  U4MODEbits.ON = 0;
  U5MODEbits.ON = 0;
  U6MODEbits.ON = 0;

  // 23. Serial Peripheral Interface
  SPI1CONbits.ON = 0;
  SPI2CONbits.ON = 0;
  SPI3CONbits.ON = 0;
  SPI4CONbits.ON = 0;

  // 24. I2C - Inter-Integrated Circuit
  I2C1CONbits.ON = 0;
  I2C2CONbits.ON = 0;
  I2C3CONbits.ON = 0;
  I2C4CONbits.ON = 0;
  I2C5CONbits.ON = 0;

  // 27. USB
  U1PWRCbits.USUSPEND = 1;      // put USB in suspend mode  PAGE 892
  U1PWRCbits.USBPWR   = 0;      // disable USB power
  U1CONbits.USBEN     = 0;      //disable entire usb module   P 902

  // 29. Real-Time Clock and Calendar
  // RTCCONbits.ON = 0;

  // 31. DMA Functions (Direct Memory Access)
  DMACONbits.ON = 0;

  // 33. Programming and Diagnostics - DDPCON Register
  DDPCONbits.JTAGEN = 0;        // disable JTAG port
  DDPCONbits.TDOEN = 0;
  DDPCONbits.TROEN = 0;

}

void main() {
  int cnt = 0;
 
  AD1PCFG = 0xFFFF;    // initialize pins RB0 Strom,RB1 Spannung & RB8 Temperatur als analog; rest ist Digital
  JTAGEN_BIT = 0;

  TRISA = 0x0000;    // as OUTPUT
  TRISB = 0x0000;
  TRISC = 0x0000;
  TRISD = 0x0000;
  TRISE = 0x0000;
  TRISG = 0x0000;

  PORTA = 0x0000;    // as OUTPUT LOW
  PORTB = 0x0000;
  PORTC = 0x0000;
  PORTD = 0x0000;
  PORTE = 0x0000;
  PORTF = 0x0000;
  PORTG = 0x0000;

  UART2_Init(9600);
  delay_ms(100);
  UART2_Write_text("\x0D\x0DuController started\xd");
  
  RTC_Init();

  EnableInterrupts();
  RTCCIE_bit = 1;

  cnt = 0;
  R_LED = 0;
  G_LED = 1;
  while(1){
  delay_ms(200);
  UART2_Write_text(".");
  LATDbits.LATD1 = ~PORTDbits.RD1;
  LATDbits.LATD2 = ~PORTDbits.RD2;
  cnt++;
  if(cnt == 10){
     cnt = 0;
     R_LED = 0;
     G_LED = 0;
     UART2_Write_text("\x0DGood Night!!!\xd");
     disablePIC32modules();
     shutDownPIC32();
     }
  }
}